#pragma once
/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/****************************************************************************
  History

$Log: game_client.cpp,v $
Revision 1.7  2007/04/10 08:22:28  ganovelli
added sun position

Revision 1.6  2007/04/09 23:08:41  ganovelli
added standalone mode

Revision 1.5  2007/03/29 21:52:44  ganovelli
added builder of circuits.

Revision 1.4  2007/03/27 23:10:29  ganovelli
added holes and tunnels to the Street
added buildings to the Circuit
added server to client transmission of each player state
added BRAKING and BOOSTING to the player state
added sun position

Revision 1.3  2007/03/21 10:34:21  ganovelli
added better steering,

Revision 1.2  2007/03/17 12:26:14  ganovelli
* pragma replaced by #ifndef directive
* Player operator= added
* bug in race_simulation.cpp corrected
* msvc2005 solutions added

Revision 1.1  2007/03/11 15:10:17  ganovelli
first working draft

Revision 1.2  2007/03/11 15:08:51  ganovelli
*** empty log message ***

Revision 1.1  2007/03/10 16:43:35  ganovelli
first working draft

****************************************************************************/

#include <pinet.h>
#include <pasync.h>

#include "messages.h"
#include "packaging.h"
#include "process_message.h"
 
#include "actions.h"
#include "game_client.h"
#include <game_viewer.h>
#include "race_simulation.h"

/// the single istance of NvmcClient
NvmcClient cli;
/// The single istance of Race
Race race;
/// The adress of the server
pt::ipaddress server_address;

/// the message queue for standalone mode
std::vector<pt::string> c2sMessages;

bool standalone_mode = false;

///
const int in_port = 8082;
///
const int out_port = 8084;
///
const int maxtoken = 4096;


///
int NvmcClient::Initialize(char * circuitFile, bool standalone){
	
	standalone_mode = standalone;

	server_address=pt::ipbcast;
	/** add a simple circuit to the list of circuits */
	circuits.push_back(Circuit());			
	
	if(circuitFile==NULL)  CreateCircularCircuit(circuits[0]); else circuits[0].Load(circuitFile);
	/** set the circuit as the circuit of the race */
	race.circuit = &circuits[0];
	race.circuit->sun_position = vcg::Point4f(10000,10000,10000,1.0);

	return 0;
};




///
int ProcessMessage(pt::string msg,pt::string host){
	const char* p = msg;
	int state_time;
	float gtime;
	vcg::Point4f & sp = race.circuit->sun_position;
	sscanf(p,"%d %f %f %f %f ",&state_time,&sp[0],&sp[1],&sp[2],&sp[3]);
	race.start_time = state_time;
	int pos =pt::pos('~',msg);
	pt::del(msg,0,pos+1);
	while(!isempty(msg)){
		ProcessMessagePlayer(msg,host);
		pos =pt::pos('~',msg);
		pt::del(msg,0,pos+1);
	}
	return 0;
}

///
class MessagesReceiverThread: public pt::thread{
public:
///
	MessagesReceiverThread():thread(false){}

///
	void execute(){
		int part = 0,cl=0;
		pt::ipmsgserver svr;
		int h = svr.bindall(in_port);
		bool quit = false;
			do
			{	
					try
					{
						++cl;
							pt::string msg = pt::lowercase(svr.receive(maxtoken));
							pt::string host = svr.get_host();
							if (pt::isempty(host))
									host = pt::iptostring(svr.get_ip());
							server_address = svr.get_ip();
							ProcessMessage(msg,host);

						if(clock()-part>1000){
							printf("RECEIV fps %d\n",cl);
							cl=0;
							part = clock();
						}
					}
					catch(pt::estream* e)
					{
							pt::perr.putf("Server error: %s\n", pt::pconst(e->get_message()));
							delete e;
					}
			} while (!quit);

	}

};


///
class MessagesSenderThread: public pt::thread{
public:
///
	MessagesSenderThread():pt::thread(false){trig = new pt::trigger(true,false);}
///
	pt::trigger * trig;
///
	void execute(){

		pt::string msg_str;
		do{
			pt::ipmessage msg(server_address,out_port);
			try
			{
				trig->wait();
			if(standalone_mode)
				ProcessMessageServer(cli.messagesToSend.back(),pt::string("localhost"));
			else
				msg.send(cli.messagesToSend.back());
			}
			catch(pt::estream* e)
			{
					server_address = pt::ipaddress(127,0,0,1);
					delete e;
			}
		}while(true);
	}
};


class RaceSimulationThread: public pt::thread {
public:
	RaceSimulationThread():pt::thread(false) {}
	void execute(){
	gTime = clock();
	int part=gTime,cl=0;
	Actions copyof_actionsToDo;
	ActionsIterator ai;
		/* first draft */
	while(true){
		++cl;
		relax(20);

		mutex_on_actionsToDo.enter();
		copyof_actionsToDo = actionsToDo;
		actionsToDo.clear();
		mutex_on_actionsToDo.leave();

		for(ai = copyof_actionsToDo.begin(); ai != copyof_actionsToDo.end(); ++ai)
			ProcessAction(*ai);

		Race::PlayersIterator pi;

		race.mutex_on_players.enter();
		Race::PlayerContainer players = race.players;
		race.mutex_on_players.leave();

		for(pi = players.begin(); pi != players.end(); ++pi) 
			Update((clock()-gTime),*pi);
		 
		race.mutex_on_players.enter();
		race.players = players;
		race.mutex_on_players.leave();


		if(clock()-part>1000){
			printf("SIMU fps %d \n",cl, cl);
			cl=0;
			part = clock();
		}

		copyof_actionsToDo.clear();
		gTime = clock();
	}
}

};
	 MessagesSenderThread s;


///
int NvmcClient::Start(){

	///* start the thread handling ipmessages reception */
	MessagesReceiverThread t;
	if(!standalone_mode)
		t.start();

	///* start the thread handling ipmessages sending */
 	s.start();

	///* start the local simulation */
	RaceSimulationThread rs;
	if(standalone_mode) rs.start();

	/* game main loop */ 
	launch_viewer(); 
	
	return 0;

};

///
int NvmcClient::Terminate(){
	return 0;
};

///
int NvmcClient::Command(C2SHeaderValue value){

//	mutex_on_messagesToSend.enter();
	messagesToSend.push_back(Pack(C2SMessage(value)));
//	mutex_on_messagesToSend.leave();

	s.trig->post();

//	mutex_on_actionsToDo.enter();
//	actionsToDo.push_back(Action(std::string("nb-ganovelli.isti.cnr.it"),value));
//	mutex_on_actionsToDo.leave();
	return 0;
}

///
void NvmcClient::UpdatePlayers(){
	race.mutex_on_players.enter();
	players = race.players;
	race.mutex_on_players.leave();
}
