/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.															*
*																			*
* lallopizzi giancarlo														*
*                                                                           *
****************************************************************************/

#include <SDL.h>
#include <gl/glew.h>
#include <GL/glu.h>
#include <vector>
#include <iostream>
#include <math.h>
#include <vcg/space/point3.h>
/*include the base definition for the vertex */
#include <vcg/simplex/vertexplus/base.h>

/*include the base definition for the face */
#include <vcg/simplex/faceplus/base.h>

/*include the base definition for the trimesh*/
#include <vcg/complex/trimesh/base.h>

/*include the algorihm that update bounding box and normals*/
#include <vcg/complex/trimesh/update/bounding.h>
#include <vcg/complex/trimesh/update/normal.h>

/*include the importer from disk*/
#include <wrap/io_trimesh/import.h>
/*include trackball*/
#include <wrap/gui/trackball.h>

/*include wrapping of the trimesh towards opengl*/
#include <wrap/gl/trimesh.h>

#include <vcg/math/Matrix44.h>


#include "myCar.h"
#include "myFuncs.h"


using namespace vcg;
using namespace std;

class CEdge;    // dummy prototype never used
class CFace;

#define MESH_CAR	"./tt.ply"


//Matrice di trasformazione delle ombre sul piano
extern GLfloat shadowMat[4][4];

extern vcg::Matrix44f myf;



/* define a vertex passing the attributes you want it to have. Each attributes has its own class.
Check vcg/simplex/vertexplus/component.h to find out the existing attributes. Note: then you could 
also personalized attributes */
class CVertex : public VertexSimp2< CVertex, CEdge, CFace, vert::Coord3f, vert::Normal3f >{};

/*same as for the vertes */ 
class CFace   : public FaceSimp2<   CVertex, CEdge, CFace, vcg::face::VertexRef, vcg::face::Normal3f, face::Color4b > {};

/*the mesh is a container of vertices and a container of faces */ 
class CMesh   : public vcg::tri::TriMesh< vector<CVertex>, vector<CFace> > {};


////////////////////////////////////////////////////////////////////////////
// Globals: the mesh, the OpenGL wrapper to draw the mesh and the trackball.
CMesh mesh;
vcg::GlTrimesh<CMesh> glCar;
vcg::Trackball track;
 
int w =1024;
int h = 768;
float d;
////////////////////////////////////////////////////////////////////////////









void initCar(){
	cout << "Sto caricando il modello dell'auto \""<<MESH_CAR<<"\"..." << endl;
	// Generic loading of the mesh from disk
	if(vcg::tri::io::Importer<CMesh>::Open(mesh,MESH_CAR)!=0) {
      printf("Error reading file \n");
			exit(0);
		}

	// Initialize the mesh itself
	vcg::tri::UpdateBounding<CMesh>::Box(mesh);      // update bounding box
	  //vcg::tri::UpdateNormals<CMesh>::PerVertexNormalizePerFaceNormalized(mesh); // update Normals
	  vcg::tri::UpdateNormals<CMesh>::PerVertexPerFace(mesh); // update Normals

	  	// Initialize the wrapper
  glCar.m = &mesh;
  glCar.Update();
}


void drawCar(vcg::Matrix44f myf){
	
	//calcolo l'inversa (trasposta) del frame della macchina
	float inv[4][4];
	unsigned int i, j;
	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			inv[i][j] = myf.ElementAt(j,i);
	
	
	glMatrixMode(GL_TEXTURE);
	glPushMatrix();
		glMultMatrixf(&inv[0][0]);

		
		glMatrixMode(GL_MODELVIEW);
		d=2.8f/mesh.bbox.Diag();
		glScale(d);
		glPushMatrix();
			//draw_axis();
			glEnable(GL_COLOR_MATERIAL);
			glCar.Draw<vcg::GLW::DMSmooth,vcg::GLW::CMPerFace,vcg::GLW::TMPerVert> ();
			glDisable(GL_COLOR_MATERIAL);
		
			//Setto la posizione e la direzione dei fari dell'auto
			float posizionedx[4]={-0.4,-1.1,0.4,1.0};
			float posizionesx[4]={0.4,-1.1,0.4,1.0};
			float direzione[4]={0,-3.0,-0.4,0.0};
			
			glLightfv(GL_LIGHT1, GL_POSITION, posizionedx);
			glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, direzione);

			glLightfv(GL_LIGHT2, GL_POSITION, posizionesx);
			glLightfv(GL_LIGHT2, GL_SPOT_DIRECTION, direzione);
		glPopMatrix();
		
			//Renderizzo quattro cubi per vedere dove ho posizionato le luci della macchina
		
		/*glPushMatrix();
			glTranslatef(posizionedx[0],posizionedx[1],posizionedx[2]);
			glScalef(0.01,0.01,0.01);
			draw_unit_cube();
		glPopMatrix();

		glPushMatrix();
			glTranslatef(posizionesx[0],posizionesx[1],posizionesx[2]);
			glScalef(0.01,0.01,0.01);
			draw_unit_cube();
		glPopMatrix();

		glPushMatrix();
			glTranslatef(posizionedx[0] + direzione[0], posizionedx[1] + direzione[1], posizionedx[2] + direzione[2]);
			glScalef(0.01,0.01,0.01);
			draw_unit_cube();
		glPopMatrix();

		glPushMatrix();
			glTranslatef(posizionesx[0] + direzione[0], posizionesx[1] + direzione[1], posizionesx[2] + direzione[2]);
			glScalef(0.01,0.01,0.01);
			draw_unit_cube();
		glPopMatrix();*/

		

		// Get ready to draw the shadow and the ground
		// First disable lighting and save the projection state
		//glPushAttrib(GL_LIGHTING_BIT);
		

		//Ripristino la matrice precedente per le texture
		glMatrixMode(GL_TEXTURE);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
	

	
}

void drawShadowCar(){
	
	glPushMatrix();
	
	// Multiply by shadow projection matrix
	glMultMatrixf((GLfloat *)shadowMat);
	
	glScale(d);
	//glTranslate(-glCar.m->bbox.Center());
	glColor4f(0.0,0.0,0.0,1.0);
	//Non so perch ci vuole questo...
	glRotatef(90,1,0,0);

	glCar.Draw<vcg::GLW::DMFlat,vcg::GLW::CMNone,vcg::GLW::TMNone> ();

	// Restore the projection to normal
	glPopMatrix();
	
}



void trackApply(bool map, vcg::Point3f center){
	
	track.center=center;
	if(map) track.radius=500;
	else track.radius= 1;

	track.GetView();
    track.Apply(false);

	glPushMatrix();
}



void trackPostDraw(){
	glPopMatrix();
	track.DrawPostApply();
}

//Resetta la trackball
void trackReset(){
	track.Reset();
}

//Gestore eventi del mouse per la trackball
void mouse_event(SDL_Event e){

	switch(e.type){
		
		case SDL_MOUSEBUTTONDOWN:

			switch(e.button.button){		
				  case SDL_BUTTON_WHEELUP:    track.MouseWheel( 1); break;
				  case SDL_BUTTON_WHEELDOWN:  track.MouseWheel(-1); break;
				  case SDL_BUTTON_LEFT:	      track.MouseDown(e.button.x, (h - e.button.y), vcg::Trackball::BUTTON_LEFT); break;
				  case SDL_BUTTON_RIGHT:	    track.MouseDown(e.button.x, (h - e.button.y), vcg::Trackball::BUTTON_RIGHT);break;
			}
			break;

		case SDL_MOUSEBUTTONUP:
			
			switch(e.button.button) {
			  case SDL_BUTTON_LEFT:	      track.MouseUp(e.button.x, (h - e.button.y), vcg::Trackball::BUTTON_LEFT); break;
			  case SDL_BUTTON_RIGHT:	    track.MouseUp(e.button.x, (h - e.button.y), vcg::Trackball::BUTTON_RIGHT);break;
			} break;

		break;
		
		case SDL_MOUSEMOTION:

			while(SDL_PeepEvents(&e, 1, SDL_GETEVENT, SDL_MOUSEMOTIONMASK));
			  track.MouseMove(e.button.x, (h - e.button.y));
			  break;

		break;


	}
}