#include <iostream>
#include <GL/glut.h>
#include <GL/glu.h>
#include <vcg/space/point3.h>


#include "game_viewer.h"

//Light positions
vcg::Point3f lightPosition(0.0f, 10000.0f,0.0f);

//Size of shadow map
const int shadowMapSize=512;

//Dimensioni viewport
extern int width,height;
//camera point of view, car position
extern vcg::Point3f pov, pos;

//Matrici di stato per calcolare la textureMatrix (cambio di frame da luce ad eye)
float lightProjectionMatrix[16];
float lightViewMatrix[16];
float cameraProjectionMatrix[16];
float cameraViewMatrix[16];


float textureMatrix[16];

//Matrice di bias per il cambio di valori da [-1,+1] a [0,1]
static float biasMatrix[16]= {0.5f, 0.0f, 0.0f, 0.0f,
							0.0f, 0.5f, 0.0f, 0.0f,
							0.0f, 0.0f, 0.5f, 0.0f,
							0.5f, 0.5f, 0.5f, 1.0f};


void setMatrixForShadow(){
	
	//Setto le due matrici di proiezione
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();

		glLoadIdentity();
		gluPerspective(90.0f, (float)width/height, 1.0f, 15000.0f);
		glGetFloatv(GL_PROJECTION_MATRIX, cameraProjectionMatrix);

		glLoadIdentity();
		gluPerspective(45.0f, 1.0f, 9900.0f, 10100.0f);
		glGetFloatv(GL_PROJECTION_MATRIX, lightProjectionMatrix);

	glPopMatrix();
	

	//Setto le due matrici di vista-modellazione
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
		glLoadIdentity();
		gluLookAt(pov.X(),pov.Y()+1,pov.Z(),pos.X(),pos.Y(),pos.Z(),0,1,0);
		glGetFloatv(GL_MODELVIEW_MATRIX, cameraViewMatrix);

		glLoadIdentity();
		gluLookAt( lightPosition.X(), lightPosition.Y(), lightPosition.Z(), 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);
		glGetFloatv(GL_MODELVIEW_MATRIX, lightViewMatrix);

	glPopMatrix();

}

void setDrawForShadow(){
	
	//Carico la matrice di proiezione per la shadow texture
	glMatrixMode(GL_PROJECTION);
    glLoadMatrixf(lightProjectionMatrix);

	//Mi metto nel punto di vista della luce
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixf(lightViewMatrix);

    //Use viewport the same size as the shadow map
    glViewport(0, 0, shadowMapSize, shadowMapSize);

}


void shadowApply(){

	//Calculate texture matrix for projection
	//This matrix takes us from eye space to the light's clip space
	//It is postmultiplied by the inverse of the current view matrix when specifying texgen	

	for(int i=0;i<4;i++){
		for(int j=0;j<4;j++){
			int element = ( i * 4 ) + j;
			textureMatrix[element] = biasMatrix[element] * lightProjectionMatrix[element] * lightViewMatrix[element];
		}
	}

}