/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.															*
*																			*
* lallopizzi giancarlo														*
*                                                                           *
****************************************************************************/

#include <SDL.h>
#include <math.h>
#include <time.h>
#include <vcg/space/point3.h>
#include <vector>

#include <iostream>
#include <GL/glew.h>
#include <GL/glut.h>
#include <GL/glu.h>
#include "myShadow.h"

using namespace std;

extern float textureMatrix[16]; 

#define TEXTURE_STREET_FILE		"../client1/roadw2.bmp"			 
#define TEXTURE_BUILDINGS_FILE	"../client1/building.bmp"
#define TEXTURE_TERRAIN_FILE	"../client5/skybox2/neg_y.bmp"
#define TEXTURE_TREE_FILE		"../client2/texture/_tree.bmp"

#define TEXTURE_SBOX_XP_FILE	"../client5/skybox2/neg_x.bmp"
#define TEXTURE_SBOX_XN_FILE	"../client5/skybox2/pos_x.bmp"
#define TEXTURE_SBOX_YP_FILE	"../client5/skybox2/pos_y.bmp"
#define TEXTURE_SBOX_YN_FILE	"../client5/skybox2/neg_y.bmp"
#define TEXTURE_SBOX_ZP_FILE	"../client5/skybox2/pos_z.bmp"
#define TEXTURE_SBOX_ZN_FILE	"../client5/skybox2/neg_z.bmp"

#define TEXTURE_STREET			1			 
#define TEXTURE_BUILDINGS		2
#define TEXTURE_TERRAIN			3
#define TEXTURE_TREE			4

#define TEXTURE_SBOX_XP         5
#define TEXTURE_SBOX_XN			6
#define TEXTURE_SBOX_YP			7
#define TEXTURE_SBOX_YN			8
#define TEXTURE_SBOX_ZP			9
#define TEXTURE_SBOX_ZN			10

#define TEXTURE_SHADOW			11

#define NUM_TOT_TEXTURES		11


//Array di textures
GLuint myTextures[NUM_TOT_TEXTURES];

//Superficie SDL
SDL_Surface *s;

const int shadowMapSize=512;

//Carico tutte le texture
void loadTexture(char tex[], int t){
	
	glBindTexture(GL_TEXTURE_2D,myTextures[t]);

	//Metodo di memorizzazione
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	//La 11  per lo shadow mapping
	if(t!=11){
		s = SDL_LoadBMP(tex);
		if(!s){
			printf("Texture non caricata");
			SDL_FreeSurface(s);
		}

		//Clamping or wrapping
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		
		//Minification, Magnification
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

		glTexImage2D(GL_TEXTURE_2D, 0,  GL_RGB, s->w, s->h, 0, GL_RGB, GL_UNSIGNED_BYTE, s->pixels);
		gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,s->w,s->h,GL_RGB, GL_UNSIGNED_BYTE, s->pixels);

		SDL_FreeSurface(s);

	}
	else{
		//Clamping or wrapping
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
		
		//Minification, Magnification
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			
		//glTexImage2D( GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, shadowMapSize, shadowMapSize, 0, GL_DEPTH_COMPONENT, GL_UNSIGNED_BYTE, NULL);

		glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, 0, 0, shadowMapSize, shadowMapSize, 0);
	
	}
	
	
}

//Switch the texture
void switchTexture(int t){
	glBindTexture(GL_TEXTURE_2D,myTextures[t]);
	
}



void init_cube_map(){
		
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	/* definiamo il wrapping delle coordinate texture per ogni direzione.
		GL_REPEAT pu essere sostituito con CL_CLAMP */
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP);

	/*definiamo come deve essere fatto il smapling nel caso di MAGNIFICATION e
	di MINIFICATION */
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

	// Carica la texture per il piano ortogonale a X su x positivo
	s = SDL_LoadBMP("../client5/skybox2/neg_x.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,s->w,s->h,GL_RGB, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);
	// Carica la texture per il piano ortogonale a X su x negativo
	s = SDL_LoadBMP("../client5/skybox2/pos_x.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,s->w,s->h,GL_RGB, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);
	// Carica la texture per il piano ortogonale a Y su y positivo
	s = SDL_LoadBMP("../client5/skybox2/pos_y.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,s->w,s->h,GL_RGB, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);
	// Carica la texture per il piano ortogonale a Y su y negativo
	s = SDL_LoadBMP("../client5/skybox2/neg_y.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,s->w,s->h,GL_RGB, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);
	// Carica la texture per il piano ortogonale a Z su z positivo
	s = SDL_LoadBMP("../client5/skybox2/pos_z.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,s->w,s->h,GL_RGB, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);
	// Carica la texture per il piano ortogonale a Z su z negativo
	s = SDL_LoadBMP("../client5/skybox2/neg_z.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,s->w,s->h,GL_RGB, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);
	
}


//Abilito il cube mapping
void enableCubeMapping(bool environment){
	
	//Abilito il cube mapping per il riflesso sulla macchina
	if(environment){
		/*stabilisci come devono essere generate le coordinate texture */
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
	}
	

	/*Abilita la generazione di coordinate texture */
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);

	/* definisci come il risultato del texturing deve influire sul colore finale del frammento.
		Provate a cambiare GL_MODULATE in GL_DECAL e togliere tutte le luci*/
	if(environment)
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	else
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	/*abilita il cube mapping */
    glEnable(GL_TEXTURE_CUBE_MAP_EXT);   
    //glEnable(GL_NORMALIZE);
}


//Disabilito il cube mapping
void disableCubeMapping(){
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_GEN_R);
	glDisable(GL_TEXTURE_CUBE_MAP_EXT);
}


//Carico tutte le texture che servono
void initTextures(){

	//find out how many texture units are available
	GLint textureunits;
	glGetIntegerv(GL_MAX_TEXTURE_UNITS,&textureunits);
	cout << "Number of texture units present: " << textureunits << endl;


	cout<<"Sto caricando le textures..." << endl;

	init_cube_map();
	cout<<"Inizializzazione cube mapping..." << endl;
	glGenTextures(NUM_TOT_TEXTURES,myTextures);
	
	
	loadTexture(TEXTURE_STREET_FILE, TEXTURE_STREET);
	loadTexture(TEXTURE_BUILDINGS_FILE, TEXTURE_BUILDINGS);
	loadTexture(TEXTURE_TERRAIN_FILE, TEXTURE_TERRAIN);
	loadTexture(TEXTURE_TREE_FILE, TEXTURE_TREE);
	cout<<"4 texture caricate..." << endl;
	loadTexture(TEXTURE_SBOX_XP_FILE, TEXTURE_SBOX_XP);
	loadTexture(TEXTURE_SBOX_XN_FILE, TEXTURE_SBOX_XN);
	loadTexture(TEXTURE_SBOX_YP_FILE, TEXTURE_SBOX_YP);
	loadTexture(TEXTURE_SBOX_YN_FILE, TEXTURE_SBOX_YN);
	loadTexture(TEXTURE_SBOX_ZP_FILE, TEXTURE_SBOX_ZP);
	loadTexture(TEXTURE_SBOX_ZN_FILE, TEXTURE_SBOX_ZN);
	cout<<"10 texture caricate..." << endl;
	loadTexture("shadow mapping", TEXTURE_SHADOW);
	cout<<"11 texture caricate... (anche shadow...!!) " << endl;
}

void makeShadow(){
	//Read the depth buffer into the shadow map texture
	glBindTexture(GL_TEXTURE_2D, myTextures[TEXTURE_SHADOW]);
    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, shadowMapSize, shadowMapSize);
	//glReadPixels(0,0,shadowMapSize,shadowMapSize,GL_DEPTH_COMPONENT,GL_UNSIGNED_BYTE,myTextures[TEXTURE_SHADOW]);
}

void shadowTextureCreate(){


	float auxTextureMatrix[16];

	//inverto la matrice per prendere le righe
	for(int i=0;i<4;i++){
		for(int j=0;j<4;j++){
			auxTextureMatrix[( j * 4 ) + i] = textureMatrix[( i * 4 ) + j];
		}
	}

	//Set up texture coordinate generation.
    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGenfv(GL_S, GL_EYE_PLANE, &auxTextureMatrix[0]);
    glEnable(GL_TEXTURE_GEN_S);

    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGenfv(GL_T, GL_EYE_PLANE, &auxTextureMatrix[4]);
    glEnable(GL_TEXTURE_GEN_T);

    glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGenfv(GL_R, GL_EYE_PLANE, &auxTextureMatrix[8]);
    glEnable(GL_TEXTURE_GEN_R);

    glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGenfv(GL_Q, GL_EYE_PLANE, &auxTextureMatrix[12]);
    glEnable(GL_TEXTURE_GEN_Q);


	//Bind & enable shadow map texture
    glBindTexture(GL_TEXTURE_2D, myTextures[TEXTURE_SHADOW]);
    

    //Enable shadow comparison
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE);

    //Shadow comparison should be true (ie not in shadow) if r<=texture
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC_ARB, GL_LEQUAL);

    //Shadow comparison should generate an INTENSITY result
    glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE_ARB, GL_INTENSITY);

	

}

