/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/****************************************************************************
  History

$Log: mycar.cpp,v $
Revision 1.18  2007/04/10 18:55:37  gian_lorenzo
FINAL RELEASE (aggiustate le normali nei palazzi)

Revision 1.17  2007/04/10 17:57:26  gian_lorenzo
FINAL RELEASE

Revision 1.16  2007/04/10 14:42:52  gian_lorenzo
*** empty log message ***

Revision 1.15  2007/04/10 14:09:53  gian_lorenzo
*** empty log message ***

Revision 1.14  2007/04/10 09:56:04  gian_lorenzo
*** empty log message ***

Revision 1.13  2007/04/06 06:47:18  gian_lorenzo
*** empty log message ***

Revision 1.12  2007/04/04 13:52:37  gian_lorenzo
*** empty log message ***

Revision 1.11  2007/04/04 09:58:04  gian_lorenzo
*** empty log message ***

Revision 1.10  2007/04/03 13:33:10  gian_lorenzo
*** empty log message ***

Revision 1.9  2007/04/03 09:32:06  gian_lorenzo
*** empty log message ***

Revision 1.8  2007/04/02 17:03:47  gian_lorenzo
*** empty log message ***

Revision 1.7  2007/03/31 14:35:55  gian_lorenzo
*** empty log message ***

Revision 1.6  2007/03/22 19:05:00  gian_lorenzo
*** empty log message ***

Revision 1.5  2007/03/22 09:00:09  gian_lorenzo
*** empty log message ***

Revision 1.4  2007/03/20 18:14:05  gian_lorenzo
*** empty log message ***

Revision 1.3  2007/03/20 16:15:25  gian_lorenzo
*** empty log message ***

Revision 1.2  2007/03/19 23:35:02  gian_lorenzo
*** empty log message ***

Revision 1.1  2007/03/19 18:51:14  gian_lorenzo
*** empty log message ***

Revision 1.2  2007/03/19 09:28:33  gian_lorenzo
*** empty log message ***

Revision 1.1  2007/03/18 18:26:50  gian_lorenzo
*** empty log message ***

Revision 1.3  2007/03/11 18:11:59  ganovelli
minor modif


****************************************************************************/
#include <SDL.h>

#include <GL/glew.h>
#include <GL/glut.h>
#include <GL/glu.h>
#include <vector>

// mesh definition
#include <vcg/simplex/vertexplus/base.h>
#include <vcg/simplex/faceplus/base.h>
#include <vcg/complex/trimesh/base.h>
#include <vcg/complex/trimesh/update/bounding.h>
#include <vcg/complex/trimesh/update/normal.h>
#include <wrap/io_trimesh/import.h>
#include <wrap/gl/trimesh.h>
#include <wrap/gui/trackball.h>
#include "mycar.h"
#include "util.h"

class CEdge;    // dummy prototype never used
class CFace;

using namespace std;
using namespace vcg;

class CVertex : public VertexSimp2< CVertex, CEdge, CFace, vert::Coord3f, vert::Normal3f >{};//face::WedgeTexture2f
class CFace   : public FaceSimp2<   CVertex, CEdge, CFace, face::VertexRef, face::Normal3f, face::Color4b> {};
class CMesh   : public vcg::tri::TriMesh< vector<CVertex>, vector<CFace> > {};

CMesh mesh;
vcg::GlTrimesh<CMesh> glWrap;

bool load_car(){
	int r=vcg::tri::io::Importer<CMesh>::Open(mesh,"../../models/cars/ka.ply");
	if(r==0){
		// Initialize the mesh itself
		vcg::tri::UpdateBounding<CMesh>::Box(mesh);      // update bounding box
		// update Normals
		vcg::tri::UpdateNormals<CMesh>::PerVertexPerFace(mesh);
		// Initialize the wrapper
		glWrap.m = &mesh;
		//glWrap.TMId.push_back(1);
		glWrap.Update();
		return true;
	}
	else
		return false;
}

void setHeadlight(vcg::Point3f direction, vcg::Point3f p){
	const GLfloat type[] = { 1.0f, 1.0f, 0.3f, 1.0 };
	direction=direction.Normalize();
	vcg::Point4f dir(direction.X(),direction.Y(),direction.Z(),1.0);
	GLfloat position[]={0.0,0.0,0.0,1.0};
	//GLfloat dir[]={0.0,-1.0,1.0,1.0};

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT1);

	glPushMatrix();
	glTranslatef(p.X(),p.Y(),p.Z());
	glLightfv(GL_LIGHT1,GL_AMBIENT_AND_DIFFUSE,type);
	glLightfv(GL_LIGHT1,GL_SPECULAR,type);
	glLightfv(GL_LIGHT1,GL_POSITION,&position[0]);
	glLightfv(GL_LIGHT1,GL_SPOT_DIRECTION,&dir[0]);
	glLighti(GL_LIGHT1,GL_SPOT_CUTOFF,15);
	glLighti(GL_LIGHT1,GL_CONSTANT_ATTENUATION,2.5);
	glLighti(GL_LIGHT1,GL_SPOT_EXPONENT,35.0);
	glPopMatrix();
}

void draw_car(){
	glMateriali(GL_FRONT, GL_SHININESS, 55);
	glPushMatrix();
    float d=mesh.bbox.Diag();
    glScale(1/d);
	glRotatef(180,0,1,0);
	glRotatef(-90,1,0,0);
	glTranslate(-glWrap.m->bbox.Center());
	//vcg::GLW::TMPerWedge
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_COLOR_MATERIAL);
	glWrap.Draw<vcg::GLW::DMSmooth, vcg::GLW::CMPerFace,vcg::GLW::TMNone> ();
	glPopMatrix();
}

GLuint compileCar(){
	GLuint CarDL=glGenLists(1);
	glNewList(CarDL,GL_COMPILE);
	draw_car();
	glEndList();
	return CarDL;
}
