#include <iostream>
#include <vector>
#include <vcg/space/point3.h>
#include <GL/glew.h>
#include <GL/glut.h>
#include <GL/glu.h>
#include <SDL.h>
#include "textfile.h"
#include "game_viewer.h"
#include "race.h"

using namespace std;

void buildPalace(vector<Vertex> &footprint, vector<vcg::Point3f> *palace, float height){
	int i=0;
	vcg::Point3f x0,x1;
	vcg::Point3f y0,y1;

	if(palace->size()>0)
		palace->clear();
	for(i=0;i<footprint.size();i++){
		x0=footprint[i].P();y0=footprint[(i+1)%footprint.size()].P();
		x1[0]=x0[0];x1[1]=x0[1]+height;x1[2]=x0[2];
		y1[0]=y0[0];y1[1]=y0[1]+height;y1[2]=y0[2];
		palace->push_back(x0);
		palace->push_back(x1);
		palace->push_back(y1);
		palace->push_back(y0);
	}
}

void drawPalace(vector<vcg::Point3f> palace){
	int i=0;
	vcg::Point3f n;
	glColor3f(0.7,0.7,0.7);
	glBindTexture(GL_TEXTURE_2D,3);
	glBegin(GL_QUADS);
	for(i=0;i<palace.size();i+=4){
		n=(palace[i+1]-palace[i])^(palace[i+2]-palace[i]);
		n=n.Normalize();
		glNormal3f(n.X(),n.Y(),n.Z());
		glTexCoord2f(0.0,5.0);
		glVertex3f(palace[i].X(),palace[i].Y()-0.45,palace[i].Z());
		glTexCoord2f(0.0,0.0);
		glVertex3f(palace[i+1].X(),palace[i+1].Y()-0.45,palace[i+1].Z());
		glTexCoord2f(5.0,0.0);
		glVertex3f(palace[i+2].X(),palace[i+2].Y()-0.45,palace[i+2].Z());
		glTexCoord2f(5.0,5.0);
		glVertex3f(palace[i+3].X(),palace[i+3].Y()-0.45,palace[i+3].Z());
	}
	glEnd();
}

// mi genero una display list per il circuito
GLuint circuitDraw(){
	int i=0,j=0,k=0;
	Street::TilesIterator ti;
	vcg::Point3f p0,p1;
	// genero l' ID della lista per il ciruito
	GLuint CircuitDL=glGenLists(1);
	glNewList(CircuitDL,GL_COMPILE);
	// draw circuit
	glActiveTexture(GL_TEXTURE0);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_COLOR_MATERIAL);
	//glMateriali(GL_FRONT, GL_SHININESS, 3.5);
	glBindTexture(GL_TEXTURE_2D,2);
	glBegin(GL_QUAD_STRIP);
	glNormal3f(0.0,1.0,0.0);
	for(ti = race.circuit->street.path.begin(); ti != race.circuit->street.path.end();ti++){
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		if(k==0){
			i=0;j=0;
			k=1;
		} else{
			i=1;j=0;
			k=0;
		}
		glTexCoord2f(i,j++);
		glVertex3f(p0[0],p0[1]-0.47,p0[2]);
		glTexCoord2f(i,j);
		glVertex3f(p1[0],p1[1]-0.47,p1[2]);
	}
	glEnd();
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_COLOR_MATERIAL);
	glEndList();
	return CircuitDL;
}

// disegno il celo
void skyInit(){
	glActiveTexture(GL_TEXTURE0);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	/* definiamo il wrapping delle coordinate texture per ogni direzione.
		GL_REPEAT pu essere sostituito con CL_CLAMP */
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_REPEAT);

	/*definiamo come deve essere fatto il smapling nel caso di MAGNIFICATION e
	di MINIFICATION */
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

	// Carica la texture per il piano ortogonale a X su x positivo
	SDL_Surface *s = SDL_LoadBMP("../../client5/skybox2/pos_x.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);
	// Carica la texture per il piano ortogonale a X su x negativo
	s = SDL_LoadBMP("../../client5/skybox2/neg_x.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);
	// Carica la texture per il piano ortogonale a Y su y positivo
	s = SDL_LoadBMP("../../client5/skybox2/pos_y.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);
	// Carica la texture per il piano ortogonale a Y su y negativo
	s = SDL_LoadBMP("../../client5/skybox2/neg_y.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);
	// Carica la texture per il piano ortogonale a Z su z positivo
	s = SDL_LoadBMP("../../client5/skybox2/pos_z.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);
	// Carica la texture per il piano ortogonale a Z su z negativo
	s = SDL_LoadBMP("../../client5/skybox2/neg_z.bmp");
	glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT, 0, GL_RGB, s->w, s->h, 0, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	SDL_FreeSurface(s);

	/*stabilisci come devono essere generate le coordinate texture */
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
}

void skyEnable(){
	/*Abilita la generazione di coordinate texture */
	glActiveTexture(GL_TEXTURE0);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_TEXTURE_GEN_S);glEnable(GL_TEXTURE_GEN_T);glEnable(GL_TEXTURE_GEN_R);
	
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
   /*abilita il cube mapping */
    glEnable(GL_TEXTURE_CUBE_MAP_EXT);
}

void skyDisable(){
	glActiveTexture(GL_TEXTURE0);
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_GEN_R);
	glDisable(GL_TEXTURE_CUBE_MAP_EXT);
}

void skyDraw(vcg::Point3f pov, float angle){
	GLfloat h=0.3;
	glActiveTexture(GL_TEXTURE0);
	glEnable(GL_TEXTURE_2D);
	glDepthMask(false);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glPushMatrix();
	glTranslatef(pov.X(),pov.Y(),pov.Z());
	glRotatef(angle*180/M_PI,0,1,0);

	glBindTexture(GL_TEXTURE_2D,100);
	glBegin(GL_QUADS);
	glNormal3f(0.0,0.0,1.0);
	glTexCoord2f(0.001,0.999);glVertex3f(-h,-h,-h);
	glTexCoord2f(0.001,0.001);glVertex3f(-h,h,-h);
	glTexCoord2f(0.999,0.001);glVertex3f(h,h,-h);
	glTexCoord2f(0.999,0.999);glVertex3f(h,-h,-h);
	glEnd();

	glBindTexture(GL_TEXTURE_2D,101);
	glBegin(GL_QUADS);
	glNormal3f(1.0,0.0,0.0);
	glTexCoord2f(0.001,0.999);glVertex3f(-h,-h,h);
	glTexCoord2f(0.001,0.001);glVertex3f(-h,h,h);
	glTexCoord2f(0.999,0.001);glVertex3f(-h,h,-h);
	glTexCoord2f(0.999,0.999);glVertex3f(-h,-h,-h);
	glEnd();

	glBindTexture(GL_TEXTURE_2D,102);
	glBegin(GL_QUADS);
	glNormal3f(-1.0,0.0,0.0);
	glTexCoord2f(0.001,0.999);glVertex3f(h,-h,-h);
	glTexCoord2f(0.001,0.001);glVertex3f(h,h,-h);
	glTexCoord2f(0.999,0.001);glVertex3f(h,h,h);
	glTexCoord2f(0.999,0.999);glVertex3f(h,-h,h);
	glEnd();

	glBindTexture(GL_TEXTURE_2D,103);
	glBegin(GL_QUADS);
	glNormal3f(0.0,-1.0,0.0);
	glTexCoord2f(0.001,0.999);glVertex3f(-h,h,-h);
	glTexCoord2f(0.001,0.001);glVertex3f(h,h,-h);
	glTexCoord2f(0.999,0.001);glVertex3f(h,h,h);
	glTexCoord2f(0.999,0.999);glVertex3f(-h,h,h);
	glEnd();

	glBindTexture(GL_TEXTURE_2D,104);
	glBegin(GL_QUADS);
	glNormal3f(0.0,1.0,0.0);
	glTexCoord2f(0.001,0.999);glVertex3f(-h,-h,-h);
	glTexCoord2f(0.001,0.001);glVertex3f(h,-h,-h);
	glTexCoord2f(0.999,0.001);glVertex3f(h,-h,h);
	glTexCoord2f(0.999,0.999);glVertex3f(-h,-h,h);
	glEnd();

	glBindTexture(GL_TEXTURE_2D,105);
	glBegin(GL_QUADS);
	glNormal3f(0.0,0.0,-1.0);
	glTexCoord2f(0.001,0.999);glVertex3f(h,-h,h);
	glTexCoord2f(0.001,0.001);glVertex3f(h,h,h);
	glTexCoord2f(0.999,0.001);glVertex3f(-h,h,h);
	glTexCoord2f(0.999,0.999);glVertex3f(-h,-h,h);
	glEnd();

	glPopMatrix();
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glDepthMask(true);
}

void loadTexture(const char* fname, int id, int type){
	SDL_Surface *s = SDL_LoadBMP(fname);

	glBindTexture(GL_TEXTURE_2D,id);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	
	if(type==0){
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		/*definiamo come deve essere fatto il smapling nel caso di MAGNIFICATION e
		di MINIFICATION */
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

		// MIPMAPPING
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, s->w, s->h, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	} else {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		// definizione del filtering della texture
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, s->w, s->h,	0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, s->pixels);
	}

	SDL_FreeSurface(s);
}

void set_shaders( char *nameV,  char *nameF, GLuint & vs, GLuint &fs,  GLuint &pr){	
	fs= glCreateShader(GL_FRAGMENT_SHADER);
	vs= glCreateShader(GL_VERTEX_SHADER);

	const char * code;
	if(nameV!=NULL){
		code = textFileRead(nameV);
		glShaderSource(vs, 1, &code,NULL);
		glCompileShader(vs);
		int errV;
		glGetShaderiv(vs,GL_COMPILE_STATUS,&errV);
		assert(errV==GL_TRUE);
	}
	if(nameF!=NULL){
		code = textFileRead(nameF);
		glShaderSource(fs, 1, &code,NULL);
		glCompileShader(fs);
		int errF;
		glGetShaderiv(fs,GL_COMPILE_STATUS,&errF);
		assert(errF==GL_TRUE);
	}
	pr = glCreateProgram();
	if(nameV!=NULL)glAttachShader(pr,vs);
	if(nameF!=NULL) glAttachShader(pr,fs);
	glLinkProgram(pr);
}

void setheadlight(){

}

void headlight(vcg::Point3f pov, vcg::Point3f pov2){
	int i=0;
	vcg::Matrix44d matrix;
	
	glActiveTexture(GL_TEXTURE1);	
	glEnable(GL_TEXTURE_2D);

	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);
	glEnable(GL_TEXTURE_GEN_Q);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glTranslatef(0.5,0.5,0);
	glScalef(0.5,0.5,1);
	gluPerspective(45,1.0,0.1,700);
	gluLookAt(pov.X(),pov.Y(),pov.Z(),pov2.X(),pov2.Y(),pov2.Z(),0,1,0);
	glGetDoublev(GL_MODELVIEW_MATRIX,&matrix[0][0]);
	glLoadIdentity();

	matrix=vcg::Transpose(matrix);

	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR );

	glTexGendv(GL_S, GL_EYE_PLANE, &matrix[0][0]);
	glTexGendv(GL_T, GL_EYE_PLANE, &matrix[1][0]);
	glTexGendv(GL_R, GL_EYE_PLANE, &matrix[2][0]);
	glTexGendv(GL_Q, GL_EYE_PLANE, &matrix[3][0]);
	glPopMatrix();
	glActiveTexture(GL_TEXTURE0);
}

void headlightDisable(){
	glActiveTexture(GL_TEXTURE1);
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_GEN_R);
	glDisable(GL_TEXTURE_GEN_Q);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE0);
}

void tree(vcg::Point3f pov){
	Circuit::TreesIterator t;

	vcg::Matrix44f m;

	vcg::Point3f r,n,up=vcg::Point3f(0.0,1.0,0.0);

	glActiveTexture(GL_TEXTURE0);	
	glEnable(GL_TEXTURE_2D);

	for(t=race.circuit->trees.begin();t!=race.circuit->trees.end();t++){
		r=(pov - t->pos);
		if(r.Norm()<700) {
			r=r.Normalize();
			up=vcg::Point3f(0.0,1.0,0.0);
			n=(r^up).Normalize();
			//up=(n^r).Normalize();

			m.SetIdentity();
			m[0][0]=n[0];m[0][1]=up[0];m[0][2]=r[0];
			m[1][0]=n[1];m[1][1]=up[1];m[1][2]=r[1];
			m[2][0]=n[2];m[2][1]=up[2];m[2][2]=r[2];
			m=vcg::Transpose(m);
			
			glPushMatrix();
			glTranslatef(t->pos.X(),t->pos.Y()-0.5,t->pos.Z());
			glMultMatrixf(&m[0][0]);

			glBindTexture(GL_TEXTURE_2D,4);
			glBegin(GL_QUADS);
			glTexCoord2f(0.0,0.0);glVertex3f(-1.0,0.0,0.0);
			glTexCoord2f(1.0,0.0);glVertex3f(-1.0,t->height,0.0);
			glTexCoord2f(1.0,1.0);glVertex3f(1.0,t->height,0.0);
			glTexCoord2f(0.0,1.0);glVertex3f(1.0,0.0,0.0);
			glEnd();
			glPopMatrix();
		}
	}
}
